@@ -3,11 +3,12 @@
 import logging
 from re import compile
 
-from django.conf import settings
+import django.conf
 from django.contrib.auth import logout
 from django.core.exceptions import ImproperlyConfigured
-from django.core.urlresolvers import reverse
-from django.http import HttpResponseRedirect, HttpResponse
+from django.core.urlresolvers import reverse, NoReverseMatch
+from django.http import HttpResponseRedirect, HttpResponse, HttpResponseServerError
+from django.test.signals import setting_changed
 from django.utils import simplejson as json, timezone
 import django.views.static
 
@@ -16,62 +17,96 @@ from password_expiry import password_is_expired
 
 logger = logging.getLogger(__name__)
 
+class BaseMiddleware(object):
+    """
+    Abstract class containing some functionality common to all middleware.
+    """
 
-class MandatoryPasswordChangeMiddleware:
+    REQUIRED_SETTINGS = ()
+    OPTIONAL_SETTINGS = ()
+
+    def load_setting(self, setting, value):
+        """
+        Called initially for each of the keys in REQUIRED_SETTINGS and OPTIONAL_SETTINGS,
+        and again whenever any of these settings change (from the setting_changed signal).
+        Passed the setting key and the new value, which may be None for the keys in
+        OPTIONAL_SETTINGS. If no setting keys are defined then this method is never called.
+        """
+
+    def _on_setting_changed(self, sender, setting, value, **kwargs):
+        if setting in self.REQUIRED_SETTINGS or setting in self.OPTIONAL_SETTINGS:
+            self.load_setting(setting, value)
+
+    def __init__(self):
+        if self.REQUIRED_SETTINGS or self.OPTIONAL_SETTINGS:
+            for key in self.REQUIRED_SETTINGS:
+                if hasattr(django.conf.settings, key):
+                    self.load_setting(key, getattr(django.conf.settings, key))
+                else:
+                    raise django.core.exceptions.ImproperlyConfigured(
+                        self.__class__.__name__ + " requires setting " + key)
+
+            for key in self.OPTIONAL_SETTINGS:
+                self.load_setting(key, getattr(django.conf.settings, key, None))
+
+            setting_changed.connect(self._on_setting_changed)
+
+
+
+class MandatoryPasswordChangeMiddleware(BaseMiddleware):
     """
     Redirects any request from an authenticated user to the password change
     form if that user's password has expired. Must be placed after
     AuthenticationMiddleware in the middleware list.
     """
 
-    def __init__(self):
-        """
-        Looks for a valid configuration in settings.MANDATORY_PASSWORD_CHANGE.
-        If there is any problem, the view handler is not installed.
-        """
-        try:
-            config = settings.MANDATORY_PASSWORD_CHANGE
-            self.password_change_url = reverse(config["URL_NAME"])
-            self.exempt_urls = [self.password_change_url
-                                ] + map(reverse, config["EXEMPT_URL_NAMES"])
-        except:
-            logger.error("Bad MANDATORY_PASSWORD_CHANGE dictionary. "
-                         "MandatoryPasswordChangeMiddleware disabled.")
-            raise django.core.exceptions.MiddlewareNotUsed
+    OPTIONAL_SETTINGS = ("MANDATORY_PASSWORD_CHANGE",)
+
+    def load_setting(self, setting, value):
+        if value and not value.has_key("URL_NAME"):
+            raise ImproperlyConfigured(MandatoryPasswordChangeMiddleware.__name__+" requires the URL_NAME setting")
+        self.settings = value
+        self.exempt_urls = [compile(url) for url in self.settings.get("EXEMPT_URLS", ())]
 
     def process_view(self, request, view, *args, **kwargs):
-        if (not request.user.is_authenticated() or
-             view == django.views.static.serve or # Mostly for testing, since
-                                                  # Django shouldn't be serving
-                                                  # media in production.
-             request.path in self.exempt_urls):
-            return
-        if password_is_expired(request.user):
-            return HttpResponseRedirect(self.password_change_url)
+        if self.settings:
+            path = request.path_info.lstrip('/')
+
+            # Check for an exempt URL before trying to resolve URL_NAME,
+            # because the reason the URL is exempt may be because a special
+            # URL config is in use (i.e. during a test) that doesn't have URL_NAME.
+            if (not request.user.is_authenticated() or
+                view == django.views.static.serve or # Mostly for testing, since
+                                                     # Django shouldn't be serving
+                                                     # media in production.
+                any(m.match(path) for m in self.exempt_urls) or
+                request.path in map(reverse, self.settings.get("EXEMPT_URL_NAMES", ()))):
+                return
+
+            password_change_url = reverse(self.settings["URL_NAME"])
+
+            if request.path == password_change_url:
+                return
+
+            if password_is_expired(request.user):
+                return HttpResponseRedirect(password_change_url)
 
 
-class NoConfidentialCachingMiddleware:
+class NoConfidentialCachingMiddleware(BaseMiddleware):
     """
     Adds No-Cache and No-Store Headers to Confidential pages
     """
 
-    def __init__(self):
-        """
-        Looks for a valid configuration in settings.MANDATORY_PASSWORD_CHANGE.
-        If there is any problem, the view handler is not installed.
-        """
-        try:
-            config = settings.NO_CONFIDENTIAL_CACHING
-            self.whitelist = config.get("WHITELIST_ON", False)
-            if self.whitelist:
-                self.whitelist_url_regexes = map(compile, config["WHITELIST_REGEXES"])
-            self.blacklist = config.get("BLACKLIST_ON", False)
-            if self.blacklist:
-                self.blacklist_url_regexes = map(compile, config["BLACKLIST_REGEXES"])
-        except Exception:
-            logger.error("Bad NO_CONFIDENTIAL_CACHING dictionary. "
-                         "NoConfidentialCachingMiddleware disabled.")
-            raise django.core.exceptions.MiddlewareNotUsed
+    OPTIONAL_SETTINGS = ("NO_CONFIDENTIAL_CACHING",)
+
+    def load_setting(self, setting, value):
+        value = value or {}
+        self.whitelist = value.get("WHITELIST_ON", False)
+        if self.whitelist:
+            self.whitelist_url_regexes = map(compile, value['WHITELIST_REGEXES'])
+        self.blacklist = value.get("BLACKLIST_ON", False)
+        if self.blacklist:
+            self.blacklist_url_regexes = map(compile, value['BLACKLIST_REGEXES'])
 
     def process_response(self, request, response):
         """
@@ -96,7 +131,7 @@ class NoConfidentialCachingMiddleware:
         return response
 
 
-class HttpOnlySessionCookieMiddleware:
+class HttpOnlySessionCookieMiddleware(BaseMiddleware):
     """
     Middleware that tags the sessionid cookie 'HttpOnly'.
     This should get handled by Django starting in v1.3.
@@ -107,7 +142,7 @@ class HttpOnlySessionCookieMiddleware:
         return response
 
 
-class XFrameOptionsDenyMiddleware:
+class XFrameOptionsDenyMiddleware(BaseMiddleware):
     """
     This middleware will append the http header attribute
     'x-frame-options: deny' to the any http response header.
@@ -121,26 +156,27 @@ class XFrameOptionsDenyMiddleware:
         return response
 
 
-class P3PPolicyMiddleware:
+class P3PPolicyMiddleware(BaseMiddleware):
     """
     This middleware will append the http header attribute
     specifying your P3P policy as set out in your settings
     """
-    def __init__(self):
-        try:
-            self.policy = settings.P3P_COMPACT_POLICY
-        except AttributeError:
-            raise django.core.exceptions.MiddlewareNotUsed
+
+    OPTIONAL_SETTINGS = ("P3P_COMPACT_POLICY",)
+
+    def load_setting(self, setting, value):
+        self.policy = value
 
     def process_response(self, request, response):
         """
         And P3P policy to the response header.
         """
-        response['P3P'] = 'policyref="/w3c/p3p.xml" CP="%s"' % self.policy
+        if self.policy:
+            response['P3P'] = 'policyref="/w3c/p3p.xml" CP="%s"' % self.policy
         return response
 
 
-class SessionExpiryPolicyMiddleware:
+class SessionExpiryPolicyMiddleware(BaseMiddleware):
     """
     The session expiry middleware will let you expire sessions on
     browser close, and on expiry times stored in the cookie itself.
@@ -154,21 +190,19 @@ class SessionExpiryPolicyMiddleware:
     session to the login page (if required).
     """
 
+    OPTIONAL_SETTINGS = ('SESSION_COOKIE_AGE', 'SESSION_INACTIVITY_TIMEOUT')
+
     # Session keys
     START_TIME_KEY = 'starttime'
     LAST_ACTIVITY_KEY = 'lastactivity'
 
-    # Get session expiry settings if available
-    if hasattr(settings, 'SESSION_COOKIE_AGE'):
-        SESSION_COOKIE_AGE = settings.SESSION_COOKIE_AGE
-    else:
-        SESSION_COOKIE_AGE = 86400  # one day in seconds
-    if hasattr(settings, 'SESSION_INACTIVITY_TIMEOUT'):
-        SESSION_INACTIVITY_TIMEOUT = settings.SESSION_INACTIVITY_TIMEOUT
-    else:
-        SESSION_INACTIVITY_TIMEOUT = 1800  # half an hour in seconds
-    logger.debug("Max Session Cookie Age is %d seconds" % SESSION_COOKIE_AGE)
-    logger.debug("Session Inactivity Timeout is %d seconds" % SESSION_INACTIVITY_TIMEOUT)
+    def load_setting(self, setting, value):
+        if setting == 'SESSION_COOKIE_AGE':
+            self.SESSION_COOKIE_AGE = value or 86400  # one day in seconds
+            logger.debug("Max Session Cookie Age is %d seconds" % self.SESSION_COOKIE_AGE)
+        elif setting == 'SESSION_INACTIVITY_TIMEOUT':
+            self.SESSION_INACTIVITY_TIMEOUT = value or 1800  # half an hour in seconds
+            logger.debug("Session Inactivity Timeout is %d seconds" % self.SESSION_INACTIVITY_TIMEOUT)
 
     def process_request(self, request):
         """
@@ -189,31 +223,29 @@ class SessionExpiryPolicyMiddleware:
             logger.debug("New session %s started: %s" % (request.session.session_key, now))
             request.session[self.START_TIME_KEY] = now
             request.session[self.LAST_ACTIVITY_KEY] = now
-            return
-
-        start_time = request.session[self.START_TIME_KEY]
-        last_activity_time = request.session[self.LAST_ACTIVITY_KEY]
-        logger.debug("Session %s started: %s" % (request.session.session_key, start_time))
-        logger.debug("Session %s last active: %s" % (request.session.session_key, last_activity_time))
-
-        # Is this session older than SESSION_COOKIE_AGE?
-        # We don't wory about microseconds.
-        SECONDS_PER_DAY = 86400
-        start_time_diff = now - start_time
-        last_activity_diff = now - last_activity_time
-        session_too_old = (start_time_diff.days * SECONDS_PER_DAY + start_time_diff.seconds >
-                SessionExpiryPolicyMiddleware.SESSION_COOKIE_AGE)
-        session_inactive = (last_activity_diff.days * SECONDS_PER_DAY + last_activity_diff.seconds >
-                SessionExpiryPolicyMiddleware.SESSION_INACTIVITY_TIMEOUT)
-
-        if (session_too_old or session_inactive):
-            logger.debug("Session %s is inactive." % request.session.session_key)
-            request.session.clear()
         else:
-            # The session is good, update the last activity value
-            logger.debug("Session %s is still active." % request.session.session_key)
-            request.session[SessionExpiryPolicyMiddleware.LAST_ACTIVITY_KEY] = now
-        return
+            start_time = request.session[self.START_TIME_KEY]
+            last_activity_time = request.session[self.LAST_ACTIVITY_KEY]
+            logger.debug("Session %s started: %s" % (request.session.session_key, start_time))
+            logger.debug("Session %s last active: %s" % (request.session.session_key, last_activity_time))
+
+            # Is this session older than SESSION_COOKIE_AGE?
+            # We don't wory about microseconds.
+            SECONDS_PER_DAY = 86400
+            start_time_diff = now - start_time
+            last_activity_diff = now - last_activity_time
+            session_too_old = (start_time_diff.days * SECONDS_PER_DAY + start_time_diff.seconds >
+                    self.SESSION_COOKIE_AGE)
+            session_inactive = (last_activity_diff.days * SECONDS_PER_DAY + last_activity_diff.seconds >
+                    self.SESSION_INACTIVITY_TIMEOUT)
+
+            if session_too_old or session_inactive:
+                logger.debug("Session %s is inactive." % request.session.session_key)
+                request.session.clear()
+            else:
+                # The session is good, update the last activity value
+                logger.debug("Session %s is still active." % request.session.session_key)
+                request.session[SessionExpiryPolicyMiddleware.LAST_ACTIVITY_KEY] = now
 
 
 # Modified a little bit by us.
@@ -244,7 +276,7 @@ class SessionExpiryPolicyMiddleware:
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
-class LoginRequiredMiddleware:
+class LoginRequiredMiddleware(BaseMiddleware):
     """
     Middleware that requires a user to be authenticated to view any page on
     the site that hasn't been white listed. (The middleware also ensures the
@@ -258,9 +290,14 @@ class LoginRequiredMiddleware:
     loaded. You'll get an error if they aren't.
     """
 
-    EXEMPT_URLS = []
-    if hasattr(settings, 'LOGIN_EXEMPT_URLS'):
-        EXEMPT_URLS += [compile(expr) for expr in settings.LOGIN_EXEMPT_URLS]
+    REQUIRED_SETTINGS = ('LOGIN_URL',)
+    OPTIONAL_SETTINGS = ('LOGIN_EXEMPT_URLS',)
+
+    def load_setting(self, setting, value):
+        if setting == 'LOGIN_URL':
+            self.login_url = value
+        elif setting == 'LOGIN_EXEMPT_URLS':
+            self.exempt_urls = [compile(expr) for expr in (value or ())]
 
     def process_request(self, request):
         if not hasattr(request, 'user'):
@@ -273,10 +310,10 @@ class LoginRequiredMiddleware:
                 login_url = request.login_url
                 next_url = None
             else:
-                login_url = settings.LOGIN_URL
+                login_url = self.login_url
                 next_url = request.path
             path = request.path_info.lstrip('/')
-            if not any(m.match(path) for m in LoginRequiredMiddleware.EXEMPT_URLS):
+            if not any(m.match(path) for m in self.exempt_urls):
                 if request.is_ajax():
                     response = {"login_url": login_url}
                     return HttpResponse(json.dumps(response), status=401,
